#!/bin/bash

# Author of this script: http://www.getlazarus.org
# This is the universal script to install fpc 3.0.0 minimal

# If you need to fix something and or want to contribute, send your 
# changes to admin at getlazarus dot org with "fpc minimal install"
# in the subject line.

function cls { 
	osascript -e 'tell application "System Events" to keystroke "k" using command down' 
}

# Prevent this script from running as root 
if [ "$(id -u)" = "0" ]; then
   echo "This script should not be run as root"
   exit 1
fi

OS_TARGET="darwin"
OS_VERSION=$(sw_vers -productVersion | awk -F "." '{print $2}')	

if [ "$OS_VERSION" -eq 7 ]; then
	echo "Detected OSX Lion"
elif [ "$OS_VERSION" -eq 8 ]; then
	echo "Detected OSX Mountain Lion"
elif [ "$OS_VERSION" -eq 9 ]; then
	echo "Detected OSX Mavericks"
elif [ "$OS_VERSION" -eq 10 ]; then
	echo "Detected OSX Yosemite"
elif [ "$OS_VERSION" -eq 11 ]; then
	echo "Detected OSX El Capitain"
elif [ "$OS_VERSION" -eq 12 ]; then
	echo "Detected OSX High Seirra Capitain"
else
	echo "This installer requires OSX 10.7 (Lion) or above"
	echo "done."
	echo 
	exit 1
fi

if ! xcode-select -p &> /dev/null ; then
	echo "Setup has detected that xcode tools are not installed"
	read -p "Press [ENTER] to install xcode tools"
	sudo xcode-select --install &> /dev/null ;
	echo "Please wait for xcode tools to install"
	echo
	sleep 10s
	read -p "After code tools are installed press [ENTER] to continue"
	echo
	if ! xcode-select -p &> /dev/null ; then
		echo "Setup has detected that xcode tools were not completely installed"
		echo "Please wait for xcode tools to install and re-run this script"
		echo "done."
		echo
		exit 1
	fi
else
	echo "Found xcode tools"	
fi
	
# Cross platform expandPath function
function expandPath() {
	if [ `uname`="Darwin" ]; then
		[[ $1 = /* ]] && echo "$1" || echo "$PWD/${1#./}";
	else
		echo $(readlink -m `$1`)
	fi
}

# Present a description of this script
cls
echo "Universal Free Pascal 3.0.4 minimal install script"
echo "--------------------------------------------------"
echo "This script will install a minimal version of"
echo "The Free Pascal Compiler version 3.0.4"
echo
echo "After install 56MB of drive space will be used"
echo
echo "This minimal version can be used as a starting point"
echo "to build other versions of the Free Pascal Compiler"
echo
echo "Or it can be used as a minimal command line compiler"
echo "where you only need the core run time library"
echo
# The default folder
FPCDIR=$HOME/Development/fpc-3.0.4

# Ask a series of questions
while true; do
	# Ask for an install location
	echo "Enter an installation folder or press return to"
	echo "accept the default install location"
	echo 
	echo -n "[$FPCDIR]: "
		read CHOICE
	echo

	# Use FPCDIR as the default
	if [ -z "$CHOICE" ]; then
		CHOICE=$FPCDIR
	fi

	# Allow for relative paths
	CHOICE=`eval echo $CHOICE`
	EXPAND=`expandPath "$CHOICE"`
	EXPAND=${EXPAND%/}

	# Allow install only under your home folder
	if [[ $EXPAND == $HOME* ]]; then
		echo "The install folder will be:"
		echo "$EXPAND"
		echo
	else
		echo "The install folder must be under your personal home folder"
		echo
		continue
	fi

	# Confirm their choice
	echo -n "Continue? (y,n): "
	read CHOICE
	echo 

	case $CHOICE in
		[yY][eE][sS]|[yY]) 
			;;
		*)
			echo "done."
			echo
			exit 1
			;;
	esac

	# If folder already exists ask to remove it
	if [ -d "$EXPAND" ]; then
		echo "Directory already exist"
		echo -n "Remove the entire folder and overwrite? (y,n): "
		read CHOICE
		case $CHOICE in
			[yY][eE][sS]|[yY]) 
				echo
				rm -rf $EXPAND
				;;
			*)
				echo
				echo "done."
				echo
				exit 1
				;;
		esac
	fi

	break
done

# Confirm their choice
echo -n "Create a compiler shortcut after install? (y,n): "
read SHORTCUT
echo 

# Create the folder
FPCDIR=$EXPAND
mkdir -p "$FPCDIR"

# Exit if the folder could not be created
if [ ! -d "$FPCDIR" ]; then
  echo "Could not create directory"
  echo
  echo "done."
  echo
  exit 1;
fi

rm -rf $FPCDIR

# Extract free pascal
TMP=/tmp/fpc
mkdir -p $TMP
tar xvf fpc-3.0.4.tar.gz -C $TMP > /dev/null
mv $TMP/fpc-3.0.4 "$FPCDIR"
rm -rf $TMP

# Create the cfg file
$FPCDIR/bin/fpcmkcfg -d basepath=$FPCDIR/lib/fpc/\$FPCVERSION -o $FPCDIR/bin/fpc.cfg

# Count this as an install
function hit() {
	if type "curl" > /dev/null; then
		curl -s -o /dev/null "$1"
	elif type "wget" > /dev/null; then
		wget -q -O /dev/null "$1"
	fi	
}

# Convert value to all lowercase
function lowerCase() {
	local result=`echo $1 | tr '[:upper:]' '[:lower:]'`
	echo $result;
}

OS=$(lowerCase `uname`)

if [ -f /etc/lsb-release ]; then
	. /etc/lsb-release
	DIST=`lowerCase $DISTRIB_ID`
elif [ $OS = "darwin" ]; then
	DIST=`sw_vers -productVersion`
else 
	DIST="unknown"
fi

hit "http://www.getlazarus.org/installed/?minimal&os=$OS&dist=$DIST"

function createShortcut() {
	# Create a terminal configuration
	TERMINAL="Free Pascal Terminal.command"
	echo "osascript -e 'tell app \"Terminal\"" > "$TERMINAL"
	echo "    do script \"export PPC_CONFIG_PATH=\\\"$FPCDIR/bin\\\" && export PATH=\\\"\$PPC_CONFIG_PATH:\$PATH\\\"\"" >> "$TERMINAL"
	echo "end tell'" >> "$TERMINAL"
	chmod +x "$TERMINAL"
}

echo "Your Free Pascal 3.0.0 minimal is now installed"
echo 
echo "To use it in any terminal include it in the path"
echo "using the following commands:"
echo 
echo "export PPC_CONFIG_PATH=$FPCDIR/bin"
echo "export PATH=\$PPC_CONFIG_PATH:\$PATH"
echo 

case $SHORTCUT in
	[yY][eE][sS]|[yY]) 
		createShortcut
		echo "Or you may simply click the shortcut file titled"
		echo "Free Pascal Terminal"
		echo 
		;;
	*)
		;;
esac

echo "Then type 'fpc' to use the compiler"
echo 
